@@ -1,4 +1,4 @@
-use crate::config::{ClientConfig, ClientServiceConfig, Config, TransportType};
+use crate::config::{ClientConfig, ClientServiceConfig, Config, ServiceType, TransportType};
 use crate::config_watcher::ServiceChange;
 use crate::helper::udp_connect;
 use crate::protocol::Hello::{self, *};
@@ -8,8 +8,8 @@ use crate::protocol::{
 };
 use crate::transport::{SocketOpts, TcpTransport, Transport};
 use anyhow::{anyhow, bail, Context, Result};
-use backoff::backoff::Backoff;
 use backoff::ExponentialBackoff;
+use backoff::{backoff::Backoff, future::retry_notify};
 use bytes::{Bytes, BytesMut};
 use std::collections::HashMap;
 use std::net::SocketAddr;
@@ -33,12 +33,9 @@ pub async fn run_client(
     shutdown_rx: broadcast::Receiver<bool>,
     service_rx: mpsc::Receiver<ServiceChange>,
 ) -> Result<()> {
-    let config = match &config.client {
-        Some(v) => v,
-        None => {
-            return Err(anyhow!("Try to run as a client, but the configuration is missing. Please add the `[client]` block"))
-        }
-    };
+    let config = config.client.as_ref().ok_or(anyhow!(
+        "Try to run as a client, but the configuration is missing. Please add the `[client]` block"
+    ))?;
 
     match config.transport.transport_type {
         TransportType::Tcp => {
@@ -150,9 +147,9 @@ impl<'a, T: 'static + Transport> Client<'a, T> {
 struct RunDataChannelArgs<T: Transport> {
     session_key: Nonce,
     remote_addr: String,
-    local_addr: String,
     connector: Arc<T>,
     socket_opts: SocketOpts,
+    service: ClientServiceConfig,
 }
 
 async fn do_data_channel_handshake<T: Transport>(
@@ -165,19 +162,15 @@ async fn do_data_channel_handshake<T: Transport>(
         ..Default::default()
     };
 
-    // FIXME: Respect control channel shutdown here
     // Connect to remote_addr
-    let mut conn: T::Stream = backoff::future::retry_notify(
+    let mut conn: T::Stream = retry_notify(
         backoff,
         || async {
-            let conn = args
-                .connector
+            args.connector
                 .connect(&args.remote_addr)
                 .await
-                .with_context(|| format!("Failed to connect to {}", &args.remote_addr))?;
-            T::hint(&conn, args.socket_opts);
-
-            Ok(conn)
+                .with_context(|| format!("Failed to connect to {}", &args.remote_addr))
+                .map_err(backoff::Error::transient)
         },
         |e, duration| {
             warn!("{:#}. Retry in {:?}", e, duration);
@@ -185,6 +178,8 @@ async fn do_data_channel_handshake<T: Transport>(
     )
     .await?;
 
+    T::hint(&conn, args.socket_opts);
+
     // Send nonce
     let v: &[u8; HASH_WIDTH_IN_BYTES] = args.session_key[..].try_into().unwrap();
     let hello = Hello::DataChannelHello(CURRENT_PROTO_VERSION, v.to_owned());
@@ -201,10 +196,16 @@ async fn run_data_channel<T: Transport>(args: Arc<RunDataChannelArgs<T>>) -> Res
     // Forward
     match read_data_cmd(&mut conn).await? {
         DataChannelCmd::StartForwardTcp => {
-            run_data_channel_for_tcp::<T>(conn, &args.local_addr).await?;
+            if args.service.service_type != ServiceType::Tcp {
+                bail!("Expect TCP traffic. Please check the configuration.")
+            }
+            run_data_channel_for_tcp::<T>(conn, &args.service.local_addr).await?;
         }
         DataChannelCmd::StartForwardUdp => {
-            run_data_channel_for_udp::<T>(conn, &args.local_addr).await?;
+            if args.service.service_type != ServiceType::Udp {
+                bail!("Expect UDP traffic. Please check the configuration.")
+            }
+            run_data_channel_for_udp::<T>(conn, &args.service.local_addr).await?;
         }
     }
     Ok(())
@@ -336,7 +337,7 @@ async fn run_udp_forwarder(
             val = s.recv(&mut buf) => {
                 let len = match val {
                     Ok(v) => v,
-                    Err(_) => {break;}
+                    Err(_) => break
                 };
 
                 let t = UdpTraffic{
@@ -427,15 +428,14 @@ impl<T: 'static + Transport> ControlChannel<T> {
         info!("Control channel established");
 
         let remote_addr = self.remote_addr.clone();
-        let local_addr = self.service.local_addr.clone();
         // Socket options for the data channel
         let socket_opts = SocketOpts::from_client_cfg(&self.service);
         let data_ch_args = Arc::new(RunDataChannelArgs {
             session_key,
             remote_addr,
-            local_addr,
             connector: self.transport.clone(),
             socket_opts,
+            service: self.service.clone(),
         });
 
         loop {
