@@ -1,6 +1,7 @@
 use crate::config::{Config, ServerConfig, ServerServiceConfig, ServiceType, TransportType};
 use crate::config_watcher::ServiceChange;
 use crate::constants::{listen_backoff, UDP_BUFFER_SIZE};
+use crate::helper::retry_notify_with_deadline;
 use crate::multi_map::MultiMap;
 use crate::protocol::Hello::{ControlChannelHello, DataChannelHello};
 use crate::protocol::{
@@ -508,13 +509,11 @@ fn tcp_listen_and_send(
     let (tx, rx) = mpsc::channel(CHAN_SIZE);
 
     tokio::spawn(async move {
-        // FIXME: Respect shutdown signal
-        let l = backoff::future::retry_notify(listen_backoff(), || async {
+        let l = retry_notify_with_deadline(listen_backoff(),  || async {
             Ok(TcpListener::bind(&addr).await?)
         }, |e, duration| {
-            error!("{:?}. Retry in {:?}", e, duration);
-        })
-        .await
+            error!("{:#}. Retry in {:?}", e, duration);
+        }, &mut shutdown_rx).await
         .with_context(|| "Failed to listen for the service");
 
         let l: TcpListener = match l {
@@ -623,17 +622,13 @@ async fn run_udp_connection_pool<T: Transport>(
 ) -> Result<()> {
     // TODO: Load balance
 
-    // FIXME: Respect shutdown signal
-    let l: UdpSocket = backoff::future::retry_notify(
+    let l = retry_notify_with_deadline(
         listen_backoff(),
-        || async {
-            Ok(UdpSocket::bind(&bind_addr)
-                .await
-                .with_context(|| "Failed to listen for the service")?)
-        },
+        || async { Ok(UdpSocket::bind(&bind_addr).await?) },
         |e, duration| {
-            warn!("{:?}. Retry in {:?}", e, duration);
+            warn!("{:#}. Retry in {:?}", e, duration);
         },
+        &mut shutdown_rx,
     )
     .await
     .with_context(|| "Failed to listen for the service")?;
